---
# A library of quantitative markers of seizure severity
## Statistical analysis
### Sarah Jane Gascoigne
### 22/04/2022

To recreate tables presented in Figures 4 and 5A/B, run 'statistical_analysis.R' (set working directory to source file location prior to running)
The following libraries are used and may need to be installed:
- readxl
- car
- stats
- DescTools
- lme4
- lmerTest
- coin
- rstatix
- symmetry
- moments
- Rfast
- cylcop

Please note that not all analyses presented in Gascoigne et al., 2023 (Epilepsia) can be performed with the data provided here to protect the confidentiality of patient data. 

See below for detailed notes on included scripts and functions:

The purpose of this project was to develop a library of interpretable, objective and quantitative markers of seizure severity from iEEG recordings. This library contains 16 markers which fall into three main branches (and one additional marker):
- **'peak' markers** (line length, energy and delta & theta & alpha & beta & low gamma & high gamma band power)
- **'spatial' markers** (proportion of channels in imprint, proportion of channels at maximum concurrent activity, time to maximum concurrent activity and proportion of seizure duration to maximum concurrent activity)
- **suppression markers** (PGES duration, partial suppression duration, suppression strength)
- seizure duration

iEEG recordings are pre-processed and used to calculate all markers in MATLAB (code available ... ). Statistical analyses are performed in R using code available in this repository. MATLAB code is also present in this repository, through which most visualisations used in paper figures are created.  
All analyses can be performed from 'statistical_analysis.R'- this pulls in all libraries and functions required. Results are stored in 'results' folder - any output to be uploaded to MATLAB for visualisation is stored in 'matlab_vis' folder. Below is a detailed explanation of each R file and the analysis performed.

## src
Folder containing scripts for (1) importing and cleaning data, (2) performing main statistical analysis and (3) performing supplementary data analysis.
functions.R loads all R libraries required and contains all functions used within this analysis.

### data_preparation.R
Reads in seizure severity, patient information and surgery outcome tables. Data is cleaned and transformed as necessary. The three tables needed for statistical analysis are maintained in the global environment.
Two .csv files are exported for visualisation in MATLAB (*severity_clean* & *pat_931_markers*).
### stat_across_patients.R
Script performs across patient validation of markers against ILAE classification of focal vs subclinical and focal vs ftbtc seizures. within this script, correlation between markerks averaged across each patient are correlated with disease duration. 
Additional supplementary analyses of validation of TLE and eTLE patients only are also performed within this script. 
### stat_within_patients.R
Script performs within patient analyses of Wilcoxon rank sum test (to validate markers against ILAE seizure type), comparing Wilcoxon r values (to determine which patient differences drive results), circular-linear correaltion (to determine if there is evidence of circadian modulation) and Spearman's rank correlation between markers and time spent in the EMU (to determine if there is evidence of longer-term modulation).
### supp_preictal_peak_markers
Script performs supplementary analysis by repeating across patinent analysis on 'peak' markers as caluculated based on *preictal* iEEG segments to determine if it is possible to determine the seizure type based on recodings in the 120 seconds before the onset of a seizure. 
### functions.R
#### preprocess_severity
Takes input of raw .xlsx files, cleans and transforms data as requires.
**Input**
  *data*             : raw severity table as imported from severity_table.xlsx 
  *patient_info*      : raw table of patient metadata as imported from patient_meta.xlsx 
  *markers*           : array of markers you're interested in (make sure they mach column names in severity_table) 
**Output**  
  *data_clean *       : seizure severity table containing all markers and patient information relevant to statistical analysis 
**Own functions used** 
  *remove_duplicates *        : Iterates through severity table and removes duplicate observations (ensuring all seizures are unique) 
  *log_transform  *           : Log transforms 'peak' and duration markers
  *relabel_ilae  *            : Relabel sz_ilae_type to be "focal", "subclin", "ftbtc" or "nan"
  *additional_variables*      : Calculates additional variables for severity table (e.g. proportion of channels at max)
  *clean_patient_data *       : Cleans and adds known patient information
  *remove_unnecessary_columns*: Reduces columns in severity_clean to only those needed for analyses

#### remove_duplicated
Iterates through severity table and removes any duplicate observations - ensuring all seizures are unique.
**Input*
  *data *             : raw severity table as imported from severity_table.xlsx
**Output **
  *data*              : severity table with only unique seizures 
 
#### log_transform
To normalise distributions of 'peak' markers and duration markers, a natural log transformation was applied.
**Input**
  *data*              : severity table (following removal of duplicates)
**Output**
  *data*              : severity table with log transformed variables
  
#### relabel_ilae
In this study, only *focal*, *subclinical* and focal to bilateral tonic clonic (*ftbtc*) seizures are considered. **NOTE**: seizures labelled as *aura* based on ILAE classification are relabelled as *focal* for the purpose of this study.
**Input**
  *data*             : severity table 
**Output**
  *data*             : severity table with relabeled ilae classification

#### additional_variables
Further variables are required that are not present in the seizure severity table extracted from matlab. This function calculates:
- *ilae_binary*           : binary variable indicating seizure type (1 for focal) 
- *prop_channels_imprint* : proportion of recording channels involved in seizure at any point (allows for comparison across patients with different numbers of channels)
- *prop_channels_max *    : proportion of recording channels involved in seizure at point of maximum concurrent activity (allows for comparison across patients with different numbers of channels)
- *hour*                  : hour of seizure onset 

**Input**
  *data*              : severity table 
**Output**
  *data*              : severity table with all variables required for statistical analysis
  
#### clean_patient_data
Cleans patient data and adds known variables not available in patient_meta.xlsx. Additional variables of patient age and disease duration at time of recording are also calculated.
**Input**
  *data*              : table of patient metadata
**Output**
  *data*              : cleaned table of patient metadata
  
#### remove_unnecessary_columns
Reduces the size of severity table by purging unnecessary columns.
**Input**
  *data*              : severity table
**Output** 
  *data*              : severity table with severity markers, seizure type, seizure times, and patient information

#### clean_surgery_outcome
We only consider surgery outcome at the 12 month point, so most data in treatment_outcome.xlsx is not relevant. This function produces a table with only three columns (patient ID, outcome as ILAE score, binary outcome (good/bad)).
**Input**
  *surgery_outcome*   : surgery outcome table as imported from treatment_outcome.xlsx 
**Output**
  *surgery_outcome*   : simplified surgery_outcome table

#### assess_models
This function creates and assesses all hierarchical logistic regression models created for all markers to validate markers against ILAE seizure classification.
**Input**
  *data*              : severity_clean table
  *markers*          : array of strings denoting severity markers to be considered
  *sz_type*          : array of two string values denoting seizure types to be considered (must be two of "focal", "subclin", "ftbtc)                       
**Output**
  *models*          : table including model assessment information for all models created
  *AUC_table*       : table of auc of roc values (used to assess model performance as classifier)
**Own functions used**
  *create_models*   : create and assess models for across patient validation against ILAE seizure type
  *calc_AUC_ROC*    : creates roc curve for a model and calculates area under curve
  *model_info*      : create and assess all models (using create models)
  

#### create_models
Function to create and assess an individual model with an option to alter the model created.
**Input**
  *ilae*              : binary array displaying seizure type (focal vs subclin or focal vs ftbtc)
  *pat*               : array of patient ids
  *marker*            : array of marker values
  *mod_number*        : Numeric variable indicating type of hierarchical logistic regression model to create
  1. fixed effect marker
  2. random effect patient
  3. fixed effect marker, random effect patient (random intercept)
  4. fixed effect marker, random effect patient (random intercept and slope)
   *output_type*       : indicator variable to determine if output is the model or model assessment
   1. model assessment (aic, bic, log likelihood, auc of roc, deviance, deviance p-value)
   2. model (to be compared)
**Output**
*model/model_info*  : model or model assessment information (depending on what is selected in input variables)
 
#### calc_AUC_ROC = function(model,ilae,pat,marker){
Create ROC curve base on 100 classification thresholds between 0 and 1, from this the area under the curve (AUC) is calculated.
**Input**
  *model*             : model as outouted by create_models function (with output_type = 2)
  *ilae*              : binary array displaying seizure type (focal vs subclin or focal vs ftbtc)
  *pat*               : array of patient ids
  *marker*            : array of marker values
**Output**
 *model_auc*         : area under roc curve for the model
  


#### model_info 
This function calculates and assesses **all** models considered in across patient validation.
**Input**
  *data*             : severity table
  *markers*          : array of markers to be considered
  *plot_assumptions* : indicator variable (1: plot marker against logit(fitted values) to check linearity,  0: do not plot)
**Output**
  *model_info_table* : table of model assessment information for all models created

#### wilcoxon_rank_sum 
Performs Wilcoxon rank sum test and store r values to validate markers against ILAE seizure type on a within-patient basis.
**Input**
   *data*              : severity table
   *patient_info*      : table of patient metadata
   *markers*           : array of strings denoting markers to be considered
   *test_assumptions*  : indicator variable stating whether or not to test symmetry about median assumption of Wilcoxon rank sum test (1: perform test, 0: do not)
   *comp_pat_included* : indicator variable stating if the function should consider patients included/removed to determine if any demographics are underrepresented (1: plot, 0: do not)
**Output**
  * wilcoxon_test_output* : matrix of r values for validating markers against ILAE seizure classification on a within-patient basis (only patients meeting inclusion criteria are included)
   *(skewness_ok)*        : matrix indicating if assumptions of Wilcoxon rank sum were satisfied (only returned if test_assumptions = 1)

#### comp_wilcoxon_eff_size 
Results suggest that effect sizes between focal and subclinical seizures are driven by patient differences rather than suggesting a 'superior' marker. Therefore, we investigated differences in  r values across different patient groups. For binary parient information (*TLE vs eTLE*, *sex*, *outcome*), Wilcoxon rank sum test was used to compare groups. For continuous variables (*age*, *disease duration*, *number of recording channels*, *number of seizures*), linear regression models were created with the following form (r value ~ continuous variable) - p-values were assessed to determine if there was a significant impact on r values. 
**Input**
  *wilcoxon_test_output*             : table created in wilcoxon_rank_sum function
  *data*                             : severity table
  *surgery_outcome*                  : table of surgery outcomes
  *patient_info *                    : patient meta data table
**Output** 
  *comp_eff_size*                    : table of Wilcoxon rank sum r values for binary variables (TLE vs eTLE, sex, surgery outcome) and p-values from linear regression models for continuous variables (disease duration, age, number of channels, number of seizures) to assess impact of disserent patient information on ability to distinguish focal and subclinical seizures.
  
#### circ_lin_perm
Performs circular linear correlation betwen markers and hour of seizure start for all patients with 5 or more seizures permutation test with n_perm iterations determines p-value.
Input
  *data*                    : severity table
  *markers*                 : array of strings denoting severity markers to consider
  *n_perm*                 : number of permutations to perform for each correlation test
**Output**
  *cor_val_clean*           : matrix of correlation values for each patient meeting inclusion criteria
  *p_val_clean*             : matrix of p values for each patient meeting inclusion criteria
  

##### summarise_seizures
Function summarises seizures to give one value per marker per patient. There is an option of how to suumarise the seizures (either *median* or *95th percentile*).
**Input**
 *data*              : Table of marker and patient information 
 *markers*           : Array of markers you're interested in (make sure they mach column names in severity_table)
 *seizure_type*      : Choose seizure type to assess ("all", "focal" or "subclin")
 *med_or_peak *      : Choose method of averaging markers ("median" = median, "peak" = 95th percentile)
**Output**
 *aver_markers*      : Data frame of seizures summarised for each patient (where possible)

#### cor_with_dd 
Function to determine correlation between averaged patient seizure (using summarise_seizures) and disease duration.
**Input**
  *data*              : Table of marker and patient information 
  *markers*          : Array of markers you're interested in (make sure they mach column names in severity_table)
  *seizure_type*      : Choose seizure type to assess ("all", "focal" or "subclinical")
  *med_or_peak*      : Choose method of averaging markers ("median" = median, "peak" = 95th percentile)
**Output**
  *corr_with_dd*      : table of Spearman rank correlation  coefficent and p-value for each marker
  
#### cor_with_dd_extended
Function to determine correlation between average patient seizure and disease duration with option to separate by diagnosis and seizure type.
**Input**
 *data*              : table of marker and patient information 
 *markers*           : array of markers you're interested in (make sure they mach column names in severity_table)
 *sz_types*          : choose seizure types to compare ("all", "focal" and/or "subclinical")
 *diagnoses*         : choose diagnoses to compare ("all", "temporal" and/or "extratemporal")
 *med_or_peak*       : choose method of averaging markers ("median" = median, "peak" = 95th percentile)
**Output**
  *cor_table*         : table of correlation values (separated by diagnosis/seizure type if desired)
  *p_table*           : table of p-values (separated by diagnosis/seizure type if desired)
  

#### corr_with_emu
Function to calculate Spearman's rank between marker values and time patient has spent in the EMU to capture longer-term modulation of seizure severity. 
**Input**
  *data*              : table of marker and patient information 
  *markers*           : array of markers you're interested in (make sure they mach column names in severity_table)
**Output** 
  *cor_val_clean*         : table of correlation values for each patient meeting inclusion criteria
  *p_val_clean*           : table of p-values for each patient meeting inclusion criteria

### AUC_bootsrapping.R
Code used to perform bootstrapping to compare AUC values against seizure duration to determine if markers show marked improvement over seizure duration in distinguishing seizure types. 
Bootstrapping procedure maintained the number of seizures of each type for each patient such that the shape of the data was unchanged. 
This analysis was performed for comparison of focal vs. subclinical seizures and focal seizures with and without loss of awareness. 
 


