### Load libraries
# Suppress "written in R version __" warnings
default_warnings = getOption("warn")
options(warn = -1)
library("readxl")
library("car")
library("stats")
library("DescTools")
library("lme4")
library("lmerTest")
library("coin")
library("rstatix")
library("symmetry")
library("moments")
library("Rfast")
library("cylcop")
# Return warnings for rest of code
options(warn = default_warnings)
rm(default_warnings)

### Data cleaning and pre-processing
## Relabel patient ids
relabel_pat = function(data, patient_info, surgery_outcome){
  patient_ids = unique(data$patient_id)
  rep_ids = patient_ids
  count_GLAS = 0
  count_UCLH = 0
  # Identify UCLH and Glasgow patients 
  for(i in 1:length(patient_ids)){
    if(grepl("G", patient_ids[i])==TRUE){
      count_GLAS = count_GLAS + 1
      rep_ids[i] = paste("G",count_GLAS, sep="")
    }else{
      count_UCLH = count_UCLH + 1
      rep_ids[i] = paste("U",count_UCLH, sep="")
    }
  }
  
  for (pat in 1:length(patient_ids)){
    data[data$patient_id == patient_ids[pat],]$patient_id = rep_ids[pat]
    patient_info[patient_info$patient_id == patient_ids[pat],]$patient_id = rep_ids[pat]
    surgery_outcome[surgery_outcome$patient_id == patient_ids[pat],]$patient_id = rep_ids[pat]
  }
  # Create table matching patient IDs 
  pat_relab = data.frame(old=patient_ids, new=rep_ids)
  
  return(list(data, patient_info, surgery_outcome, pat_relab))
}

## Preprocess seizure severity table
preprocess_severity = function(data, patient_info, markers){
  # Preprocess and clean seizure severity table
  # Sarah Jane Gascoigne
  # 10/04/22
  
  
  # Input
  #   data              : raw severity table as imported from severity_table.xlsx
  #   patient_info      : raw table of patient metadata as imported from patient_meta.xlsx
  #   markers           : array of markers you're interested in (make sure they mach column names in severity_table)
  
  # Output 
  #   data_clean        : seizure severity table containing all markers and patient information relevant to statistical analysis
  
  # Functions used
  #   remove_duplicates         : Iterates through severity table and removes duplicate observations (ensuring all seizures are unique)
  #   log_transform             : Log transforms 'peak' and duration markers
  #   relabel_ilae              : Relabel sz_ilae_type to be "focal", "subclin", "ftbtc" or "nan"
  #   additional_variables      : Calculates additional variables for severity table (e.g. proportion of channels at max)
  #   clean_patient_data        : Cleans and adds known patient information
  #   remove_unnecessary_columns: Reduces columns in severity_clean to only those needed for analyses
  
  ## Data cleaning
  # Remove duplicate observations from severity table
  data = remove_duplicates(data)
  
  ## Transformations
  # We want to log-transform 'peak' markers and durations
  data = log_transform(data)
  
  # Relabel ilae seizure classification
  data = relabel_ilae(data)
  
  # Calculate necessary variables not included in raw table 
  #   (ilae_binary, prop_channels_imprint, prop_channels_max, prop_to_max, time)
  data = additional_variables(data)
  
  # Clean patient_info table
  patient_info = clean_patient_data(patient_info)
  
  # Merge data with patient information
  data = merge(data, patient_info, on='patient_id')
  # NOTE: Patient information is only available for UCLH patients
  
  # Remove columns not relevant to analyses
  data_clean = remove_unnecessary_columns(data, markers)
  
  return(data_clean)
}

## Remove duplicate observations in severity table
remove_duplicates = function(data){
  # Iterates through severity table and removes duplicate observations (ensuring all seizures are unique)
  # Sarah Jane Gascoigne
  # 10/04/22
  
  
  # Input
  #   data              : raw severity table as imported from severity_table.xlsx
  
  # Output 
  #   data              : severity table with only unique seizures 
  
  data_temp = data
  data = data.frame(matrix(ncol = ncol(data_temp), nrow = 0))
  colnames(data) = colnames(data_temp)
  for (i in 1:dim(data_temp)[1]){
    if(data_temp$db_ind[i] %in% data$db_ind){
      print(sprintf("Seizure %s is duplicate",data$db_ind[i]))
    }else{
      data = rbind(data,data_temp[i,])
    }
  }
  # Check for duplicate times
  n_occur = data.frame(table(data$start))
  duplicate_table = n_occur[n_occur$Freq>1,]
  duplicate_ids = c()
  for(dup in 1:nrow(duplicate_table)){
    duplicates = data[factor(data$start) %in% duplicate_table[dup,1],]
    duplicate_ids = c(duplicate_ids, duplicates$segment_id[-1])
  }
  # Remove duplicates
  data = data[!data$segment_id %in% duplicate_ids,]
  
  return(data)
}

## Log transform peak and duration markers
log_transform = function(data){
  # Transforms 'peak' markers and any durations
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data              : severity table (following removal of duplicates)
  
  # Output 
  #   data              : severity table with log transformed variables
  
  ## Transformations
  # We want to log-transform 'peak' markers and durations
  # Create array of columns to be log transformed (peak and duration markers)
  log_col = c(grep('dur', colnames(data), value=TRUE), 
              grep('top', colnames(data), value=TRUE))
  log_col_id = match(log_col,colnames(data))
  
  # Log transform
  data[,log_col_id] = log(data[,log_col_id] )
  # In duration markers, some had duration 0, so log gives -Inf, relabel to nan
  data[data==-Inf] = NaN
  return(data)
}

## Relabel ilae seizure classification
relabel_ilae = function(data){
  # Relabels ilae_sz_type to be "focal_aware", "focal_unaware", "focal_unknown", subclin", "ftbtc" or "na"
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data              : severity table 
  
  # Output 
  #   data              : severity table with relabeled ilae classification
  
  # Relabel sg seizures to ftbtc
  data[data$ilae_sz_type == 'sg',]$ilae_sz_type = 'ftbtc'
  # One seizure labelled as 'subclinical' rather than subclin
  data[data$ilae_sz_type=="subclinical",]$ilae_sz_type = 'subclin'
  data[data$ilae_sz_type=="na",]$ilae_sz_type = 'nan'
  return(data)
}

## Calculate additional variables
additional_variables = function(data){
  # Calculates following variables necessary for statistical analysis
  # ilae_binary           : binary variable indicating seizure type (1 for focal) 
  # prop_channels_imprint : proportion of recording channels involved in seizure at any point (allows for comparison across patients with different numbers of channels)
  # prop_channels_max     : proportion of recording channels involved in seizure at point of maximum concurrent activity (allows for comparison across patients with 
  #                         different numbers of channels)
  # time                  : time of seizure onset given as hours since midnight (not integers)
  
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data              : severity table 
  
  # Output 
  #   data              : severity table with all variables required for statistical analysis
  
  
  # Create variable to indicate seizure type (to use in logistic regression)
  # data$ilae_binary = ifelse(data$ilae_sz_type=='focal',1,0)
  data$ilae_binary <- as.factor(ifelse(data$ilae_sz_type=='subclin', 0,
                                       ifelse(data$ilae_sz_type=='focal_aware', 1, 
                                              ifelse(data$ilae_sz_type=='focal_unaware', 2, 
                                                     ifelse(data$ilae_sz_type=='ftbtc', 3, NaN)))))
  
  
  # Calculate the proportion of channels included in the seizure
  if("num_chan_imprint" %in% colnames(data)){
    data$prop_channels_imprint = data$num_chan_imprint/data$num_total_chan
  }
  
  # Calculate the proportion of channels at point of maximum involvement
  if("num_chan_max_concurrent" %in% colnames(data)){
    data$prop_channels_max = data$num_chan_max_concurrent/data$num_total_chan
  }
  
  # Calculate the proportion of the seizure duration required to reach max
  if("time_till_max_concurrent" %in% colnames(data)){
    data$prop_to_max = data$time_till_max_concurrent/data$duration
  }
  
  # Extract the time of seizure start in hours (to be used in periodic regression)
  # This will include hour, minute and second but is recorded as hours
  hour = as.numeric( format(as.POSIXct(data$start), format = "%H"))
  minute = as.numeric( format(as.POSIXct(data$start), format = "%M"))
  second = as.numeric( format(as.POSIXct(data$start), format = "%S"))
  # Time of seizure (in hours)
  data$time = hour + (minute/60) + (second/3600)
  
  # Relabel focal seizures
  data[data$ilae_sz_type == "aura",]$ilae_sz_type = 'focal'
  data[data$ilae_sz_type == "focal" & !is.na(data$loss_of_awareness) & data$loss_of_awareness == 1 ,]$ilae_sz_type = 'focal_unaware'
  data[data$ilae_sz_type == "focal" & data$loss_of_awareness == 0 & !is.na(data$loss_of_awareness),]$ilae_sz_type = 'focal_aware'
  data[data$ilae_sz_type == "focal" & is.na(data$loss_of_awareness),]$ilae_sz_type = 'focal_unknown'
  data[data$ilae_sz_type == "focal" & data$loss_of_awareness == "NaN",]$ilae_sz_type = 'focal_unknown'
  
  
  return(data)
  
}

## Clean patient metadata
clean_patient_data = function(data){
  # Cleans patient data and adds known variables not available in patient_meta.xlsx
  # Calculates the following variables
  # age               : age of patient at time of recording
  # disease_duration  : years since epilepsy diagnosis at time of recording
  
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data              : table of patient metadata
  
  # Output 
  #   data              : cleaned table of patient metadata
  
  # Calculate approximate age of patient at time of recording
  data$age = data$recording_year - data$birth_year
  
  # Calculate approximate duration of epilepsy (in years) at time of recording
  data$disease_duration = data$age - data$age_onset
  # Rename ID column to patient_id to allow for merging data
  if("ID" %in% colnames(patient_info)){
    names(data)[names(data) == "ID"] = "patient_id"
  }
  
  # One patient was diagnosed and recorded in the same year, their disease duration appears as
  # -1, change this to zero
  data[data$patient_id==592,]$disease_duration = 0
  # Patient GLAS038 is missing DoB but we know disease duration is 9y
  data[data$patient_id == "GLAS038",]$disease_duration = 9
  
  
  return(data)
}

## Remove variables not necessary for analyses
remove_unnecessary_columns = function(data, markers){
  # Removes columns from severity_table which are not relevant to statistical analysis (to reduce size of table).
  
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data              : severity table 
  
  # Output 
  #   data              : severity table with severity markers, seizure type, seizure times, and patient information
  
  # Columns of severity table we need to keep
  columns = c("segment_id","patient_id","ilae_sz_type", "ilae_binary", markers, "start", "time",
              "is_TLE", "sex", "age", "disease_duration", "num_total_chan", "baseline_awake")
  
  ## Remove unnecessary variables from severity table
  # Identify columns to keep 
  keep_columns = match(columns,colnames(data))
  # Retain only columns with information relevant to  further analyses
  data_clean = data[,keep_columns]
  
  return(data_clean)
}

## Clean surgery outcome data
clean_surgery_outcome = function(surgery_outcome){
  # Cleans surgery outcome table to contain only patient_id, treatment oucome (ILAE score) and binary good/bad outcome variable
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   surgery_outcome             : surgery outcome table as imported from treatment_outcome.xlsx 
  
  # Output 
  #   surgery_outcome             : simplified surgery_outcome table
  surgery_outcome = surgery_outcome[,match(c("ID_Jane", "treatment_outcome"),colnames(surgery_outcome))]
  names(surgery_outcome)[names(surgery_outcome) == "ID_Jane"] = "patient_id"
  # Change surgery outcome such that ILAE 1 or 2 is good (1) and ILAE3+ is bad (0)
  surgery_outcome$outcome_binary = ifelse(surgery_outcome$treatment_outcome<3,1,0)
  return(surgery_outcome)
}

### Across patient validation

## Create and assess all hierarchical logistic regression models for all markers
assess_models = function(data, markers, sz_type){
  # Create and assess models
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data             : severity_clean table
  #   markers          : array of strings denoting severity markers to be considered
  #   sz_type          : array of two string values denoting seizure types to be considered
  #                       (must be two of "focal", "subclin", "ftbtc)                       
  
  # Output 
  #   models          : table including model assessment information for all models created
  #   AUC table       : table of auc of roc values (used to assess model performance as classifier)
  
  # Functions used
  # create_models   : create and assess models for across patient validation against ILAE seizure type
  # calc_AUC_ROC    : creates roc curve for a model and calculates area under curve
  # model_info      : create and assess all models (using create models)
  
  
  
  if(length(sz_type)!=2){
    cat("State two seizure types (subclin, focal, focal_aware, focal_unaware, ftbtc)")
    cat("Select two of subclin, focal, focal_aware, focal_unaware and ftbtc (other seizure types not supported)")
  }else{
    # Pull out seizures of the types to be compared
    severity_table = data[data$ilae_sz_type == sz_type[1]|data$ilae_sz_type == sz_type[2],]
    # severity_table$ilae_binary =  ifelse(severity_table$ilae_sz_type = sz_type[1], 1, 0)
    # Create models
    models = model_info(severity_table, markers, 0) 
    # Remove AUC values if deviance is unsuitable
    tryCatch({models[models$p_value<0.05 & !is.nan(models$p_value),]$AUC_ROC = NaN}, error=function(w) cat(""))
    # Create table of AUC values
    AUC_table = matrix(models$AUC_ROC, ncol =4, byrow=TRUE)
    colnames(AUC_table) = c("RP","FM","RI","RIS")
    rownames(AUC_table) = markers
    
    # Return tables
    return(list(models,AUC_table))
  }
  
  
}

## Create and assess models for across patient validation against ILAE seizure type
create_models = function(ilae,pat,marker,mod_number,output_type){
  # Create and assess models
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   ilae              : binary array displaying seizure type (focal vs subclin or focal vs ftbtc)
  #   pat               : array of patient ids
  #   marker            : array of marker values
  #   mod_number        : Numeric variable indicating type of hierarchical logistic regression model to create
  # 1 : Fixed effect marker
  # 2 : Random effect patient
  # 3 : Fixed effect marker, random effect patient (random intercept)
  # 4 : Fixed effect marker, random effect patient (random intercept and slope)
  #   output_type       : indicator variable to determine if output is the model or model assessment
  # output_type 1 : model assessment (aic, bic, log likelihood, auc of roc, deviance, deviance p-value)
  # output_type 2 : model (to be compared)
  
  # Output 
  #   model/model_info  : model or model assessment information
  
  # fixed marker
  if(mod_number==1){
    # Create model
    model = glm(ilae~marker, family='binomial')
    #Extract information
    model_info = c(model$aic,BIC(model),as.numeric(logLik(model)),
                   calc_AUC_ROC(model,ilae,pat,marker),model$deviance,
                   1-pchisq(model$deviance,model$df.residual))
    # random patient
  }else if(mod_number==2){
    # Create model
    model = tryCatch({glmer(ilae~(1|pat), family='binomial')}, error=function(w) cat(""))
    if(!is.null(model)){
      # Calculate auc
      AUC_val = tryCatch({calc_AUC_ROC(model,ilae,pat,marker)}, error=function(w) cat(""))
      if(is.null(AUC_val)){
        AUC_val = NaN
      }
      #Extract information
      model_info = c(AIC(model),BIC(model),as.numeric(logLik(model)),
                     AUC_val,deviance(model),1-pchisq(deviance(model),df.residual(model)))
    }else{
      model_info = rep(NaN,7)
    }
    
    #Extract information
    model_info = c(AIC(model),BIC(model),as.numeric(logLik(model)),
                   calc_AUC_ROC(model,ilae,pat,marker),deviance(model),
                   1-pchisq(deviance(model),df.residual(model)))
    # fixed marker, random patient (random intercept)
  }else if(mod_number==3){
    # Create model
    model = tryCatch({glmer(ilae~marker+(1|pat), family='binomial')}, error=function(w) cat(""))
    if(!is.null(model)){
      # Calculate auc
      AUC_val = tryCatch({calc_AUC_ROC(model,ilae,pat,marker)}, error=function(w) cat(""))
      if(is.null(AUC_val)){
        AUC_val = NaN
      }
      #Extract information
      model_info = c(AIC(model),BIC(model),as.numeric(logLik(model)),
                     AUC_val,deviance(model),1-pchisq(deviance(model),df.residual(model)))
    }else{
      model_info = rep(NaN,7)
    }
    
    # fixed marker, random patient (random intercept and slope)
  }else if(mod_number==4){
    # Create model - this model may not converge, so we use a tryCatch to handle errors
    model = tryCatch({glmer(ilae~scale(marker) +(scale(marker)|pat), family='binomial',
                            control=glmerControl(optimizer="Nelder_Mead",optCtrl=list(maxfun=2e4)))}, error=function(w) cat(""))
    if(!is.null(model)){
      # Calculate auc
      AUC_val = tryCatch({calc_AUC_ROC(model,ilae,pat,marker)}, error=function(w) cat(""))
      if(is.null(AUC_val)){
        AUC_val = NaN
      }
      
      #Extract information
      model_info = c(AIC(model),BIC(model),as.numeric(logLik(model)),
                     AUC_val,deviance(model),1-pchisq(deviance(model),df.residual(model)))
    }
    else{
      model_info = rep(NaN,7)
    }
    
    
  }
  if(output_type == 1){
    return(model_info)
  }else if(output_type == 2){
    return(model)
  }
}

## Create roc curve and calculate auc
calc_AUC_ROC = function(model,ilae,pat,marker){
  # Create roc curve and calculate auc
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   model             : model as outouted by create_models function (with output_type = 2)
  #   ilae              : binary array displaying seizure type (focal vs subclin or focal vs ftbtc)
  #   pat               : array of patient ids
  #   marker            : array of marker values
  
  
  # Output 
  #   model_auc         : area under roc curve for the model
  
  # Set thresholds to be tested (here I am using values from 0 to 1 
  # with 0.01 spacing)
  thresh = seq(from = 0.01, to = 0.99, by = 0.01)
  tpr = vector(length=length(thresh)+2)
  fpr = tpr
  feat_df = data.frame(ilae,pat,marker)
  for(i in 1:length(thresh)){
    pred = predict(model, feat_df, type="response")
    pred_sz_type = as.numeric(ifelse(pred > thresh[i], 1, 0))
    conf_mat = table(Observed=feat_df$ilae, Predicted=pred_sz_type)
    # We can only assess ROC when the model predicts at least one of each seizure type
    if(dim(conf_mat)[1]==2 & dim(conf_mat)[2]==2){
      tpr[i+1]=conf_mat[2,2]/(conf_mat[2,2]+conf_mat[2,1])
      fpr[i+1]=conf_mat[1,2]/(conf_mat[1,2]+conf_mat[1,1])
    }else{
      if(i<length(thresh)/2){
        fpr[i] = 1
        tpr[i] = 1
      }else{
        fpr[i] = 0
        tpr[i] = 0
      }
      
    }
  }
  
  model_auc= suppressWarnings(AUC(fpr,tpr,method="trapezoid"))
  return(model_auc)
}

## Create logistic mixed effects models 
model_info = function(data,markers, plot_assumptions){
  # Create and assess logistic mixed effects models
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data             : severity table
  #   markers          : array of markers to be considered
  #   plot_assumptions : indicator variable (1: plot marker against logit(fitted values) to check linearity,  0: do not plot)
  
  
  # Output 
  #   model_info_table : table of model assessment information for all models created
  
  
  # Create logistic mixed effects models and outputs table of model information
  # NOTE : models which do not converge will have values replaced with NaN
  Models = c("Random Patient","Fixed Marker",
             "Random Intercept", "Random Intercept and Slope")
  model_info_table = data.frame(Marker=rep(markers, each = 4),
                                Model=rep(Models,times=length(markers)),
                                AIC = rep(NaN,each=4*length(markers)),
                                BIC = rep(NaN,each=4*length(markers)),
                                Log_Likelihood = rep(NaN,each=4*length(markers)),
                                AUC_ROC = rep(NaN,each=4*length(markers)),
                                Deviance = rep(NaN,each=4*length(markers)),
                                p_value = rep(NaN,each=4*length(markers)))
  count_mod = 1
  # Determine the columns the markers are stored in 
  marker_columns = match(markers,colnames(data))
  
  # Iterate through markers, creating models and storing output 
  for(i in 1:length(markers)){
    cat("\n",markers[i])
    mark = marker_columns[i]
    # Remove any seizures with missing information for this marker
    severity_mark = data[!is.na(data[,mark]),]
    # # Remove these observations of zero from the model
    # severity_mark = severity_mark[severity_mark[,mark]>0,]
    # Remove outlier seizures
    outliers = boxplot(severity_mark[,mark],plot=FALSE)
    severity_mark = severity_mark[which(!severity_mark[,mark] %in% outliers$out),]
    
    # Isolate the information we need to create the models
    ilae = ifelse(severity_mark$ilae_sz_type == severity_mark$ilae_sz_type[1],1,0)
    # ilae = severity_mark$ilae_binary
    pat = severity_mark$patient_id
    marker = severity_mark[,mark]
    
    for(model_num in 1:4){
      cat("\n \t",Models[model_num])
      tryCatch({model_info_table[count_mod+model_num-1,c(3:8)] = create_models(ilae,pat,marker,model_num,1)}, warning=function(w) cat(":\t MODEL DOES NOT CONVERGE"))
    }
    
    
    
    
    # Test mixed effect logistic regression assumptions using ggplot
    if(plot_assumptions == 1){
      plot_df = data.frame(Marker = marker,
                           Pred = logit(fitted.values(model_3)),
                           Patient = pat)
      print(ggplot( data = plot_df, aes(x=Marker, y=Pred, 
                                        group=Patient, color=Patient)) +
              geom_line()+ ggtitle(sprintf("%s Random intercept",Markers[i])))
      
      plot_df = data.frame(Marker = marker,
                           Pred = logit(fitted.values(model_4)),
                           Patient = pat)
      print(ggplot( data = plot_df, aes(x=Marker, y=Pred, 
                                        group=Patient, color=Patient)) +
              geom_line()+ ggtitle(sprintf("%s Random intercept and slope",Markers[i])))
    }
    
    count_mod = count_mod+4
    
  }
  return(model_info_table)
}

### Within patient validation

## Perform wilcoxon rank sum test for each patient meeting inclusion criteria
wilcoxon_rank_sum = function(data, patient_info, markers, test_assumptions, comp_pat_included){
  # Perform Wilcoxon rank sum test and store r values to validate marekrs against ILAE seizure type on a within-patient basis
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data              : severity table
  #   patient_info      : table of patient metadata
  #   markers           : array of strings denoting markers to be considered
  #   test_assumptions  : indicator variable stating whether or not to test symmetry about median 
  #                          assumption of Wilcoxon rank sum test (1: perform test, 0: do not)
  #   comp_pat_included : indicator variable stating if the function should consider patients 
  #                          included/removed to determine if any demographics are underrepresented (1: plot, 0: do not)
  
  
  # Output 
  #   wilcoxon_test_output : matrix of r values for validating markers against ILAE seizure classification on a within-patient basis
  #                           (only patients meeting inclusion criteria are included)
  #   (skewness_ok)        : matrix indicating if assumptions of Wilcoxon rank sum were satisfied (only returned if test_assumptions = 1)
  
  Patients = unique(data$patient_id)
  
  # As distributions are not normal, we will be using Wilcoxon rank sun test
  
  # Create empty data frames to store output
  
  wilcoxon_test_output = data.frame(matrix(nrow = length(Patients), ncol = (length(markers)+1)))
  wilcoxon_sig = data.frame(matrix(nrow = length(Patients), ncol = (length(markers)+1)))
  
  colnames(wilcoxon_test_output) = c("patient_id",markers)
  colnames(wilcoxon_sig) = c("patient_id",markers)
  marker_columns = match(markers,colnames(data))
  
  if(test_assumptions == 1){
    # Create data frames to store skewness of distribution for both focal and subclin
    # seizures - this will be used to test symmetry assumption
    skewness_focal = wilcoxon_test_output
    skewness_subclin = wilcoxon_test_output
  }
  
  
  for(pat in 1:length(Patients)){
    subject_id = Patients[pat]
    cat("\n", subject_id)
    pat_df = data[data$patient_id==subject_id,]
    for(mark in 1:length(marker_columns)){
      test_df = data.frame(Sz_type = pat_df$ilae_sz_type,Marker = pat_df[,marker_columns[mark]])
      sz_type = unique(pat_df$ilae_sz_type)
      if(length(sz_type) == 1){
        sz_type = c(sz_type, NaN)
      }
      test_df = test_df[!is.na(test_df$Marker),]
      
      # Determine if patient has enough seizures to meet inclusion criteria
      
      
      if(dim(test_df[test_df$Sz_type ==sz_type[1],])[1]>1 & 
         dim(test_df[test_df$Sz_type ==sz_type[2],])[1]>1 &
         dim(test_df)[1]>4){
        
        wilcox_out = wilcox_effsize(formula = Marker~Sz_type, data = test_df)
        wilcoxon_test_output[pat,mark+1] = as.numeric(wilcox_out$effsize[1])
        wilcoxon_sig[pat,mark+1] = tryCatch(wilcox.test(formula = Marker~Sz_type, data = test_df[test_df$Sz_type != "nan",], exact = FALSE)$p.value, error=function(w) cat("") )
        
        if(test_assumptions == 1){
          skewness_focal[pat,mark+1] = skewness(pat_df[pat_df$ilae_sz_type == "focal",marker_columns[mark]])
          skewness_subclin[pat,mark+1] = skewness(pat_df[pat_df$ilae_sz_type == "subclin",marker_columns[mark]])
        }
        
      }else{
        wilcoxon_test_output[pat,mark+1] = NaN
        wilcoxon_sig[pat,mark+1] = NaN
        if(test_assumptions == 1){
          skewness_focal[pat,mark+1] = NaN
          skewness_subclin[pat,mark+1] = NaN
        }
        
        
      }
      
    }
  }
  
  # Determine if Wilcoxon rank sum test was possible
  wilcoxon_test_output$wilcox_poss = ifelse(wilcoxon_test_output[,2]<=0, 0, 1) 
  wilcoxon_test_output$patient_id = Patients
  wilcoxon_sig$patient_id = Patients
  
  
  # Merge with patient information to determine if there are relationships
  # in those who could and could not use Wilcoxon
  wilcox_pat_info = merge(wilcoxon_test_output[,c(1,length(markers)+2)], patient_info, by='patient_id')
  
  # if(comp_pat_included == 1){
  #   par(mfrow=c(2,2))
  #   # TLE
  #   barplot(table(wilcox_pat_info[c(2,5)]), beside = TRUE,
  #           main="is TLE?", xlab="",
  #           ylab="is TLE?",col = c("red", "steelblue"))
  #   
  #   # Sex
  #   barplot(table(wilcox_pat_info[c(2,7)]), beside = TRUE,
  #           main="Sex", xlab="",
  #           ylab="Sex",col = c("red", "steelblue"))
  #   
  #   # Age
  #   boxplot(wilcox_pat_info$age~wilcox_pat_info$wilcox_poss,
  #           main="Age", xlab="Wilcoxon possible?",
  #           ylab="Age", col=c("red", "steelblue"))
  #   
  #   # Disease Duration
  #   boxplot(wilcox_pat_info$disease_duration~wilcox_pat_info$wilcox_poss,
  #           main="Disease Duration", xlab="Wilcoxon possible?",
  #           ylab="Disease Duration", col=c("red", "steelblue"))
  # }
  
  
  if(test_assumptions == 1){
    skewness_focal$patient_id = Patients
    skewness_subclin$patient_id = Patients
    # Test symmetry assumption
    skewness_focal = skewness_focal[wilcoxon_test_output$wilcox_poss == 1,]
    skewness_subclin = skewness_subclin[wilcoxon_test_output$wilcox_poss == 1,]
    skewness_ok = skewness_focal
    
    for(i in 1:nrow(skewness_focal)){
      for(j in 1:(ncol(skewness_focal)-1)){
        if(isTRUE(skewness_focal[i,j+1] < 0.5 && skewness_focal[i,j+1]>-0.5 &&
                  skewness_subclin[i,j+1] < 0.5 &&
                  skewness_subclin[i,j+1]>-0.5)==TRUE){
          skewness_ok[i,j+1] = 1 
        }else{
          skewness_ok[i,j+1] = 0
        }
        
      }
    }
  }
  # Remove any patients where tests could not be completed
  wilcoxon_test_output = wilcoxon_test_output[wilcoxon_test_output$wilcox_poss == 1,]
  wilcoxon_sig = wilcoxon_sig[wilcoxon_sig$patient_id %in% wilcoxon_test_output$patient_id,]
  # Reorder table so highest r values are at the top
  wilcoxon_test_output  = wilcoxon_test_output[order(wilcoxon_test_output$top_ll, decreasing = TRUE),]
  wilcoxon_sig = wilcoxon_sig[match(wilcoxon_test_output$patient_id, wilcoxon_sig$patient_id),]
  # Merge output with patient information
  wilcoxon_test_output = merge(wilcoxon_test_output[1:(length(markers)+1)], patient_info, on='patient_id')
  
  if(test_assumptions == 1){
    output = list(wilcoxon_test_output, wilcoxon_sig, skewness_ok)
  }else{
    output = list(wilcoxon_test_output, wilcoxon_sig)
  }
  return(output)
  
}

## Compare effect sizes across different patient groups
comp_wilcoxon_eff_size = function(wilcoxon_test_output, data, surgery_outcome, patient_info){
  # Compare r values across varied patient information
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   wilcoxon_test_output             : table created in wilcoxon_rank_sum function
  #   data                             : severity table
  #   surgery_outcome                  : table of surgery outcomes
  #   patient_info                     : patient meta data table
  
  # Output 
  #   comp_eff_size                    : table of Wilcoxon rank sum r values for binary variables (TLE vs eTLE, sex, surgery outcome)
  #                                      and p-values from linear regression models for continuous variables (disease duration, age, number of channels, number of seizures)
  #                                      to assess impact of disserent patient information on ability to distinguish focal and subclinical seizures
  
  
  # Check the number of channels in each of the patients with valid Wilcoxon tests
  wilcoxon_patients = unique(wilcoxon_test_output$patient_id)
  chan_num = c(length = length(wilcoxon_patients))
  for (i in 1:length(wilcoxon_patients)){
    chan_num[i] = data[data$patient_id==wilcoxon_patients[i],]$num_total_chan[1]
  }
  # Add number of recording channels to Wilcoxon output
  wilcoxon_test_output$chan_num = chan_num
  
  # Check the number of recorded seizures in each of the patients with valid Wilcoxon tests
  n_sz = c(length = length(wilcoxon_patients))
  for (i in 1:length(wilcoxon_patients)){
    n_sz[i] = dim(data[data$patient_id==wilcoxon_patients[i],])[1]
  }
  # Add number of recording channels to Wilcoxon output
  wilcoxon_test_output$n_sz = n_sz
  
  # Merge with patient information to check if there are any patterns
  wilcoxon_pat_info = merge(wilcoxon_test_output,patient_info, on = "patient_id" )
  
  # Does TLE/non-TLE impact effect size?
  wilcoxon_is_tle = wilcoxon_pat_info[wilcoxon_pat_info$is_TLE!="na",]
  
  # Use Wilcoxon rank sum test to compare effect sizes for sex and tle 
  sex_eff_size = rep(NaN,1,length(markers))
  sex_eff_size_sig = sex_eff_size
  for(mark in 1:length(markers)){
    test_df = data.frame(Marker = wilcoxon_is_tle[,mark+1], sex = wilcoxon_is_tle$sex)
    tryCatch({sex_eff_size[mark] = wilcox_effsize(formula = Marker~sex, data = test_df)$effsize}, error=function(w) cat(""))
    tryCatch({sex_eff_size_sig[mark] = wilcox.test(formula = Marker~sex, data = test_df, exact = FALSE)$p.value}, error=function(w) cat("") )
  }
  
  tle_eff_size = rep(NaN,1,length(markers))
  tle_eff_size_sig = tle_eff_size
  for(mark in 1:length(markers)){
    test_df = data.frame(Marker = wilcoxon_is_tle[,mark+1], tle = wilcoxon_is_tle$is_TLE)
    tryCatch({tle_eff_size[mark] = wilcox_effsize(formula = Marker~tle, data = test_df)$effsize}, error=function(w) cat(""))
    tryCatch({tle_eff_size_sig[mark] = wilcox.test(formula = Marker~tle,data = test_df, exact = FALSE)$p.value}, error=function(w) cat("") )
    
  }
  
  outcome_eff_size = rep(NaN,1,length(markers))
  outcome_eff_size_sig = outcome_eff_size
  for(mark in 1:length(markers)){
    outcome_df = merge(wilcoxon_is_tle, surgery_outcome[,c(1,3)], on='patient_id')
    test_df = data.frame(Marker = outcome_df[,mark+1], outcome = outcome_df$outcome_binary)
    tryCatch({outcome_eff_size[mark] = wilcox_effsize(formula = Marker~outcome, data = test_df)$effsize}, error=function(w) cat(""))
    tryCatch({outcome_eff_size_sig[mark] = wilcox.test(formula = Marker~outcome, data = test_df, exact = FALSE)$p.value}, error=function(w) cat("") )
  }
  
  comp_eff_size = data.frame(Marker = colnames(wilcoxon_is_tle)[2:17], Eff_size_sex = sex_eff_size,
                             Eff_size_tle = tle_eff_size, Eff_size_outcome = outcome_eff_size)
  comp_eff_size_sig = data.frame(Marker = colnames(wilcoxon_is_tle)[2:17], p_sex = sex_eff_size_sig,
                                 Eff_size_tle = tle_eff_size_sig, p_outcome = outcome_eff_size_sig)
  
  # Test association between disease duration and effect size using regression model
  disease_dur_p_val =  rep(NaN,1,length(markers))
  age_p_val =  rep(NaN,1,length(markers))
  n_chan_p_val =  rep(NaN,1,length(markers))
  n_sz_p_val =  rep(NaN,1,length(markers))
  
  for(mark in 1:length(markers)){
    if(sum(!is.nan(wilcoxon_pat_info[,mark+1])) > 0){
      # Disease duration
      disease_dur_mod = lm(wilcoxon_pat_info[,mark+1]~wilcoxon_pat_info$disease_duration)
      tryCatch({disease_dur_p_val[mark] = summary(disease_dur_mod)$coefficients[2,4]}, error=function(w) cat(""))
      
      # Age
      age_mod = lm(wilcoxon_pat_info[,mark+1]~wilcoxon_pat_info$age)
      tryCatch({age_p_val[mark] = summary(age_mod)$coefficients[2,4]}, error=function(w) cat(""))
      
      # Number of recording channels
      n_chan_mod = lm(wilcoxon_pat_info[,mark+1]~wilcoxon_pat_info$chan_num)
      tryCatch({n_chan_p_val[mark] = summary(n_chan_mod)$coefficients[2,4]}, error=function(w) cat(""))
      
      
      # Number of seizures 
      n_sz_mod = lm(wilcoxon_pat_info[,mark+1]~wilcoxon_pat_info$n_sz)
      tryCatch({n_sz_p_val[mark] = summary(n_sz_mod)$coefficients[2,4]}, error=function(w) cat(""))
      
    }else{
      disease_dur_p_val[mark] = NaN
      age_mod[mark] = NaN
      n_chan_mod[mark] = NaN
      n_sz_mod[mark] = NaN
      
    }
  }
  regr_p_val = data.frame(Marker = markers, Disease_dur = disease_dur_p_val,
                          Age = age_p_val, N_chan = n_chan_p_val, N_sz = n_sz_p_val)
  
  
  
  comp_eff_size = data.frame(Marker = markers, Eff_size_sex = sex_eff_size,
                             Eff_size_tle = tle_eff_size, Eff_size_outcome = outcome_eff_size,
                             P_val_disease_dur = disease_dur_p_val,P_val_age = age_p_val,
                             P_val_n_chan = n_chan_p_val, P_val_n_sz = n_sz_p_val)
  
  return(list(comp_eff_size, comp_eff_size_sig))
}

## Perform Wilcoxon rank sum tests
wilcoxon_analysis = function(data, patient_info, markers, sz_type){
  # Perform Wilcoxon Rank sum test comparing markers for two seizure types within patients
  # Sarah Jane Gascoigne
  # 25/07/22
  
  
  # Input
  #   data              : clean seizure severity table
  #   patient_info      : table of patient metadata
  #   markers           : array of strings denoting markers to be considered
  #   sz_type           : vector of length 2 stating seizure types to compare
  
  # Output 
  #   wilcoxon_export   : table of Wilcoxon Rank Sum r values for all patients meeting inclusion criteria (with patient information)
  
  # Functions used
  #  wilcoxon_rank_sum: performs wilcoxon rank sum test for all patients meeting inclusion criteria
  
  
  if(length(sz_type)!=2){
    cat("State two seizure types (subclin, focal_aware, focal_unaware, ftbtc)")
  }else if(all(sz_type %in% c("subclin", "focal", "focal_aware", "focal_unaware", "ftbtc")) == FALSE){
    cat("Select two of subclin, focal_aware, focal_unaware and ftbtc (other seizure types not supported)")
  }else{
    cat("\n Wilcoxon rank sum test")
    cat("\n", sz_type[1], " vs. ", sz_type[2])
    severity_sz_type = data[data$ilae_sz_type == sz_type[1]| data$ilae_sz_type == sz_type[2],]
    # Complete Wilcoxon Rank Sum Test for each patient
    
    wilcoxon_output = wilcoxon_rank_sum(severity_sz_type, patient_info, markers, 0, 1)
    wilcoxon = wilcoxon_output[[1]]
    wilcoxon_sig = wilcoxon_output[[2]]
    ## Create CSVs which will be used to create figures
    # Create csv of effect sizes (this will be used to create figure in Matlab)
    wilcoxon_tle = wilcoxon[wilcoxon$is_TLE=="temporal",]
    wilcoxon_etle = wilcoxon[wilcoxon$is_TLE=="extratemporal",]
    wilcoxon_tle = wilcoxon_tle[order(-wilcoxon_tle$top_ll),]
    wilcoxon_etle = wilcoxon_etle[order(-wilcoxon_etle$top_ll),]
    
    wilcoxon_export = rbind(wilcoxon_tle, wilcoxon[wilcoxon$is_TLE=="na",]  , wilcoxon_etle)
  }
  output = list(wilcoxon_output, wilcoxon_sig)
  return(output)
}

# Perform circular-linear correlation with permutaion test
circ_lin_perm = function(data, markers, n_perm){
  # Perform circular linear correlation between markers and seizure start for all patients with 5 or more seizures
  #   permutation test with n_perm iterations determines p-value
  # Sarah Jane Gascoigne
  # 10/04/22
  
  # Input
  #   data                    : severity table
  #   markers                 : array of strings denoting severity markers to consider
  #   n_perm                  : number of permutations to perform for each correlation test
  
  # Output 
  #   cor_val_clean           : matrix of correlation values for each patient meeting inclusion criteria
  #   p_val_clean             : matrix of p values for each patient meeting inclusion criteria
  
  
  
  patients = unique(data$patient_id)
  cor_val = matrix(NaN, ncol = length(markers), nrow = length(patients))
  p_val_perm = cor_val
  for(pat in 1:length(patients)){
    cat("\n", patients[pat])
    pat_df = data[data$patient_id == patients[pat],]
    # Only include patient if they have at least 5 seizures
    if(dim(pat_df)[1]>19){
      hour_rad = pi*(pat_df$time)/12
      for(mark in 1:length(markers)){
        cat("\n \t", markers[mark])
        m = as.matrix(pat_df[,match(markers[mark],colnames(pat_df))])
        m_clean = m[!is.na(m)]
        if(length(m_clean)>4){
          hour_rad_clean = hour_rad[!is.na(m)]
          cor_val[pat,mark] = cor_cyl(hour_rad_clean, m_clean )
          
          # Permutation test with 1000 permutations to calculate p-value
          set.seed(7)
          n = length(m_clean)
          cor_perm = vector(length = n_perm)
          for(i in 1: n_perm ){
            m_perm = sample(m_clean, n, replace = FALSE, prob = NULL)
            cor_perm[i] = cor_cyl(hour_rad_clean, m_perm)
          }
          p_val_perm[pat,mark] = sum(cor_perm>cor_val[pat,mark])/n_perm 
          
        }
      }
      
    }
    
  }
  # Replace cor_val of Inf with NaN
  cor_val[cor_val==Inf] = NaN
  
  # Give table row and column names
  colnames(p_val_perm) = markers
  colnames(cor_val) = markers
  rownames(p_val_perm) = patients
  rownames(cor_val) = patients
  
  # Remove patients with too few seizures
  cor_val_clean = cor_val[!is.nan(cor_val[,1]),]
  p_val_clean = p_val_perm[!is.nan(cor_val[,1]),]
  
  output = list(cor_val_clean, p_val_clean)
  return(output)
}

# Summarise seizures for each patient
summarise_seizures = function(data, markers, seizure_type, med_or_peak){
  # Function to summarise seizures for each patient
  # Sarah Jane Gascoigne
  # 24/03/2022
  
  
  # Input
  #   data              : Table of marker and patient information 
  #   markers           : Array of markers you're interested in (make sure they mach column names in severity_table)
  #   seizure_type      : Choose seizure type to assess ("all", "focal" or "subclin")
  #   med_or_peak       : Choose method of averaging markers ("median" = median, "peak" = 95th percentile)
  
  # Output 
  #   aver_markers      : Data frame of seizures summarised for each patient (where possible)
  
  
  # Determine all unique patient ids in data
  patients = unique(data$patient_id)
  
  # Find column indeces for marker values, seizure type and disease duration
  marker_columns = match(c(markers, "disease_duration","ilae_sz_type"),colnames(data))
  
  # Create data frame to store averaged patient seizures
  aver_markers = data.frame(matrix(nrow = length(patients),ncol = length(markers)+2))
  
  # Add column names
  colnames(aver_markers) = c("patient_id", 
                             colnames(data)[marker_columns[1:(length(markers)+1)]])
  
  # Iterate through each patient and summarise seizures as specified in function
  for(pat in 1:length(patients)){
    pat_df = data[data$patient_id == patients[pat], c(marker_columns)]
    
    # Isolate chosen seizure type
    if(seizure_type == "focal"){
      pat_df = pat_df[pat_df$ilae_sz_type == "focal",]
    }else if(seizure_type == "subclin"){
      pat_df = pat_df[pat_df$ilae_sz_type == "subclin",]
    }
    
    # Calculate average patient seizures
    # All values NaN if patient has no seizures 
    if(dim(pat_df)[1]== 0 ){
      aver_markers[pat,2:(length(markers)+2)] = rep(NaN, times = 17)
    }else{
      if(med_or_peak == "median"){
        aver_markers[pat,2:(length(markers)+2)] = apply(pat_df[,1:(length(markers)+1)], 2 ,median, na.rm = TRUE)
        
      }else if(med_or_peak == "peak"){
        aver_markers[pat,2:(length(markers)+2)] =apply(pat_df[,1:(length(markers)+1)], 2, function(x) quantile(x, probs = .95, na.rm = TRUE))
      }else{
        errorCondition("Method of summarising seizures not supported, choose median or peak (95th percentile)")
      }
    }
    
  }
  
  # Add patient id to aver_markers
  aver_markers[,1] = patients
  
  # Remove patients with no seizures 
  aver_markers = aver_markers[!is.nan(aver_markers[,2]),]
  
  # Add in patient information of TLE/eTLE
  aver_markers = unique(merge(aver_markers, data[,match(c("is_TLE"),colnames(data))], on='patient_id'))
  return(aver_markers)
}

# Assess correlation between markers and disease duration
cor_with_dd = function(data, markers, seizure_type, med_or_peak){
  # Function to determine correlation between average patient seizure and disease duration
  # Sarah Jane Gascoigne
  # 19/04/2022
  
  # Input
  #   data              : Table of marker and patient information 
  #   markers           : Array of markers you're interested in (make sure they mach column names in severity_table)
  #   seizure_type      : Choose seizure type to assess ("all", "focal" or "subclinical")
  #   med_or_peak       : Choose method of averaging markers ("median" = median, "peak" = 95th percentile)
  
  # Output 
  #   corr_with_dd      : table of Spearman rank correlation  coefficent and p-value for each marker
  
  # Calculate markers summarised across seizures (using summarise_seizures for simplicity)
  aver_markers = summarise_seizures(data, markers, seizure_type, med_or_peak)
  
  # Identify marker columns
  marker_columns = match(markers, colnames(data))
  
  # Create data frame to contain correlation output
  cor_with_dd = data.frame(Marker = colnames(data)[marker_columns[1:length(markers)]],
                           Spearman_rho = rep(NaN, 16), p_val = rep(NaN, 16))
  # Iterate through markers
  for(mark in c(1:length(markers))){
    test = cor.test(aver_markers[,mark+1],aver_markers$disease_duration, method = "spearman", exact=FALSE)
    cor_with_dd[mark,c(2,3)] = c(test$estimate,test$p.value)
  }
  
  
  return(cor_with_dd)
}

# Assess correlation between markers and disease duration (separated by diagnosis and seizure type)
cor_with_dd_extended = function(data, markers, sz_types, diagnoses, peak_or_med){
  # Function to determine correlation between average patient seizure and disease duration with option to separate by diagnosis and seizure type
  # Sarah Jane Gascoigne
  # 19/04/2022
  
  # Input
  #   data              : table of marker and patient information 
  #   markers           : array of markers you're interested in (make sure they mach column names in severity_table)
  #   sz_types          : choose seizure types to compare ("all", "focal" and/or "subclinical")
  #   diagnoses         : choose diagnoses to compare ("all", "temporal" and/or "extratemporal")
  #   med_or_peak       : choose method of averaging markers ("median" = median, "peak" = 95th percentile)
  
  # Output 
  #   cor_table         : table of correlation values (separated by diagnosis/seizure type if desired)
  #   p_table           : table of p-values (separated by diagnosis/seizure type if desired)
  
  # Create table to store output
  cor_table = data.frame(matrix(nrow = length(markers), ncol = length(sz_types)*length(diagnoses)))
  rownames(cor_table) = markers
  for(diag in 1:length(diagnoses)){
    for(sz_type in 1:length(sz_types)){
      colnames(cor_table)[((diag - 1)*length(diagnoses))+sz_type] = paste(c("Diagnosis:" ,diagnoses[diag],", Seizure Type: ", sz_types[sz_type]), collapse ="")
    }
    
  }
  
  p_table = cor_table
  # Check that the diagnoses selected to be separated are supported by the function
  if(all(diagnoses %in% c("all", "temporal", "extratemporal"))==TRUE){
    # Iteraate through selected diagnoses and pull out data from relevant patients
    for(diag in 1:length(diagnoses)){
      if(diagnoses[diag] == "all"){
        data_clean = data
      }else{
        data_clean = data[data$is_TLE == diagnoses[diag],]
      }
      # For all seizure types chosen, perform correlation analysis and store output
      for(sz in 1:length(sz_types)){
        column = ((diag - 1)*length(diagnoses))+sz
        corr_output = cor_with_dd(data_clean, markers, sz_types[sz], peak_or_med)
        cor_table[,column] = corr_output[,2]
        p_table[,column] = corr_output[,3]
      }
    }
    
    
  }
  
  return(list(cor_table, p_table))
}

# Calculate Spearman's rank correlation between markers and time in the EMU
corr_with_emu = function(data, markers){
  # calculate Spearman's rank between marker values and time patient has spent in the EMU 
  # Sarah Jane Gascoigne
  # 19/04/2022
  
  # Input
  #   data              : table of marker and patient information 
  #   markers           : array of markers you're interested in (make sure they mach column names in severity_table)
  
  # Output 
  #   cor_val_clean         : table of correlation values for each patient meeting inclusion criteria
  #   p_val_clean           : table of p-values for each patient meeting inclusion criteria
  
  # Create array of patient ids
  patients = unique(data$patient_id)
  # Identify marker columns in data 
  marker_columns = match(markers,colnames(data))
  # Correlate severity with time of recording -> Repeat this for each patient
  cor_val = matrix(ncol = length(markers),nrow = length(patients))
  p_val = matrix(ncol = length(markers),nrow = length(patients))
  for(pat in 1:length(patients)){ 
    patient = patients[pat]
    patient_data = data[data$patient_id==patient,]
    # Sort patient data so seizures are in chronological order
    patient_data = patient_data[order(patient_data$start),]
    # Only perform correlation for patients with at least 5 seizures
    if(dim(patient_data)[1]>19){
      time_from_start = c();
      for(sz in 1:dim(patient_data)[1]){
        time_from_start[sz] = difftime(patient_data$start[sz], patient_data$start[1], units="hours")
      }
      for (mark in 1:length(markers)){
        if(dim(patient_data[!is.nan(patient_data[,marker_columns[mark]]),])[1]>19){
          tryCatch({corr_test = cor.test(time_from_start,patient_data[,marker_columns[mark]], method = "spearman", exact = FALSE)
          cor_val[pat,mark] = corr_test$estimate
          p_val[pat,mark] = corr_test$p.value}, error = function(x) cat(""))
        }
        
      }
    }
    
  } 
  
  colnames(cor_val) = markers
  rownames(cor_val) = patients
  colnames(p_val) = markers
  rownames(p_val) = patients
  
  # Remove patients with too few seizures
  cor_val_clean = cor_val[!is.na(cor_val[,1]),]
  p_val_clean = p_val[!is.na(cor_val[,1]),]
  return(list(cor_val_clean, p_val_clean))
}


